package Clustericious::Config::Plugin;

use strict;
use warnings;
use Clustericious::Config::Helpers;
use Carp ();

# ABSTRACT: Deprecated module
our $VERSION = '1.14'; # VERSION


# Hack to keep old versions of Test::Clustericious::Cluster working
*EXPORT = \@Clustericious::Config::Helpers::EXPORT;

sub Clustericious::Config::Helpers::cluster
{
  goto &Clustericious::Config::Plugin::cluster;
}

Carp::carp "Clustericious::Config::Plugin is deprecated and will be removed on or after January 31 2016";

if(scalar caller eq 'Test::Clustericious::Cluster')
{
  Carp::carp "Upgrade your version of Test::Clustericious::Cluster, otherwise it will stop working when Clustericious::Config::Plugin is removed";
}


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Clustericious::Config::Plugin - Deprecated module

=head1 VERSION

version 1.14

=head1 DESCRIPTION

This is module is deprecated, please see
L<Clustericious::Config::Helper>.

=head1 AUTHOR

Original author: Brian Duggan

Current maintainer: Graham Ollis E<lt>plicease@cpan.orgE<gt>

Contributors:

Curt Tilmes

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by NASA GSFC.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
